// ScriptCommand_smartgetsizes:
// Revision:    v1.0.2 (js)
// (c) 2014 steje
//
// This is a script for written for Directory Opus v11.
// See http://www.gpsoft.com.au/redirect.asp?page=scripts for development information.

// =====================================================================================
// =========================   SCRIPT COMMAND: SmartGetSizes   =========================
//
// This script adds a 'SmartGetSizes' RAW command to Directory Opus.
//
// The command is adapted from a solution Jonathan Potter wrote for a user who asked for
// a way to make the normal 'GetSizes' command work normally for any folders selected
// without deselecting them on 'first' use, but would then subsequently calculate sizes
// for all remaining folders in the current tab if used a 'second' time.
//
// That clever code has been modified here both for the purposes of being used by an
// event handling script called OnAfterFolderChange_smartgetsizes, as well as to provide
// an additional RAW command option (GETALL/S) to the functionality of the script function
// Jon had originally devised. The GETALL simply forces size calculation for ALL folders
// upon 'first' use (without first calculating sizes for ~only the selected folders by
// ignoring the size calculation state of any selected folders)...


// Called by Directory Opus to initialize the script
function OnInit(initData)
{
   // Set DEBUG flag below to true in order to enable logging messages to the Opus Output Window
   initData.config.DEBUG = false;

   // Provide basic information about the Script
   initData.name = "ScriptCommand_smartgetsizes";
   initData.desc = "Adds the 'SmartGetSizes' command to Directory Opus.";
   initData.copyright = "(c) 2014 steje";
   initData.version = "v1.0.2 (js)";
   initData.default_enable = true;

   // Initialise the command that this script adds
   var cmd = initData.AddCommand();
   cmd.name = "SmartGetSizes";
   cmd.icon = "getsizes";
   cmd.method = "OnSmartGetSizes";
   cmd.desc = "Calculates folder sizes for selected folders on first use, then all remaining folders on " +
              "second use.\n\n" +
              "Supports the use of a GETALL option to force size calculation of all folders, ignoring folder " +
              "selection state.";
   cmd.label = "SmartGetSizes";
   cmd.template = "GETALL/S,NODESELECT/S";

   return false;
}


// Implement the SmartGetSizes command
function OnSmartGetSizes(commandData)
{
   fGetAll = false;
   fNoDeselect = false;

   // Check for the GETALL option
   if ( commandData.func.args.got_arg.getall ) fGetAll = true;

   // Check for the NODESELECT option
   if ( commandData.func.args.got_arg.nodeselect ) fNoDeselect = true;

   // If the GETALL option was NOT used, check to see if any folders are selected that have NOT already had their size calc'd
   if (!fGetAll)
   {
      // Set the flag to get all folder sizes right away since that's what we want if all selected folders have already had
      // their sizes calc'd. If any selected folders have NOT had their size calc'd we set the flag to false again and exit...
      fGetAll = true;

      var objEnum = new Enumerator(commandData.func.command.files);
      if (!objEnum.atEnd())
      {
         // Build an array of the selected items to check if any are folders that have not already had sizes calc'd
         var objFiles = new Array();
         i = 0;
         while (!objEnum.atEnd())
         {
            objFiles[i] = objEnum.item();
            ++i;
            objEnum.moveNext();
         }

         if (objFiles.length > 0)
         {
            for (i =0; i < objFiles.length; i++)
            {
               if ((objFiles[i].is_dir) && (!objFiles[i].got_size))
               {
                  logMsg("At least one folder without its size calculated was selected");
                  fGetAll = false;
                  break;
               }
            }
         }
      }
   }

   if (fGetAll) commandData.func.command.ClearFiles;
   if (fNoDeselect) commandData.func.command.deselect = false;
   commandData.func.command.RunCommand("GetSizes");

}


// Subroutine to allow toggling of the global DEBUG variable to control whether logging is performed or not
function logMsg(message)
{
   if (Script.config.DEBUG) DOpus.OutputString(message);

   return;
}
